==> sql-limiter-2.2.0.tgz: package/src/strategies/limit.js <==
const createToken = require("../create-token");
const {
  findParenLevelToken,
  findLimitInsertionIndex,
  nextNonCommentNonWhitespace,
} = require("../token-utils");

function has(tokens, startingIndex) {
  const limitKeywordToken = findParenLevelToken(
    tokens,
    startingIndex,
    (token) => token.type === "keyword" && token.value === "limit"
  );

  if (!limitKeywordToken) {
    return null;
  }

  const nextNonWC = nextNonCommentNonWhitespace(
    tokens,
    limitKeywordToken.index + 1
  );

  if (!nextNonWC) {
    throw new Error("Unexpected end of statement");
  }

  if (nextNonWC.type !== "number") {
    throw new Error(`Expected number got ${nextNonWC.type}`);
  }

  return nextNonWC;
}

/**
 * Adds limit to query that does not have it
 * @param {*} queryTokens
 * @param {*} statementKeywordIndex
 * @param {*} targetParenLevel
 * @param {*} limit
 */
function add(queryTokens, statementKeywordIndex, targetParenLevel, limit) {
  // Limit was not found, so figure out where it should be inserted
  // If last keyword is offset, need to put limit before that
  // If not offset, put limit at end, before terminator if present
  const insertBeforeToken = findParenLevelToken(
    queryTokens,
    statementKeywordIndex,
    (token) =>
      token.type === "keyword" &&
      (token.value === "offset" || token.value === "for")
  );
  if (insertBeforeToken) {
    const firstHalf = queryTokens.slice(0, insertBeforeToken.index);
    const secondhalf = queryTokens.slice(insertBeforeToken.index);
    return [
      ...firstHalf,
      createToken.keyword("limit"),
      createToken.singleSpace(),
      createToken.number(limit),
      createToken.singleSpace(),
      ...secondhalf,
    ];
  }

  // If there is a terminator add it just before
  const terminatorToken = findParenLevelToken(
    queryTokens,
    statementKeywordIndex,
    (token) => token.type === "terminator"
  );
  if (terminatorToken) {
    const firstHalf = queryTokens.slice(0, terminatorToken.index);
    const secondhalf = queryTokens.slice(terminatorToken.index);
    return [
      ...firstHalf,
      createToken.singleSpace(),
      createToken.keyword("limit"),
      createToken.singleSpace(),
      createToken.number(limit),
      ...secondhalf,
    ];
  }

  // No terminator. Append to end
  // skipping past any trailing comments, whitespace, terminator
  const targetIndex = findLimitInsertionIndex(queryTokens, targetParenLevel);
  const firstHalf = queryTokens.slice(0, targetIndex);
  const secondhalf = queryTokens.slice(targetIndex);
  return [
    ...firstHalf,
    createToken.singleSpace(),
    createToken.keyword("limit"),
    createToken.singleSpace(),
    createToken.number(limit),
    ...secondhalf,
  ];
}

exports.has = has;
exports.add = add;

==> sql-limiter-3.0.0.tgz: package/src/strategies/limit.js <==
const createToken = require("../create-token");
const {
  findParenLevelToken,
  findLimitInsertionIndex,
  nextNonCommentNonWhitespace,
} = require("../token-utils");

function has(tokens, startingIndex) {
  const limitKeywordToken = findParenLevelToken(
    tokens,
    startingIndex,
    (token) => token.type === "keyword" && token.value === "limit"
  );

  if (!limitKeywordToken) {
    return null;
  }

  // Supported LIMIT syntaxes
  // LIMIT <limit_number>                   (most common)
  // LIMIT <offset_number>,<limit_number>   (supported by ClickHouse)

  const firstNumber = nextNonCommentNonWhitespace(
    tokens,
    limitKeywordToken.index + 1
  );

  if (!firstNumber) {
    throw new Error("Unexpected end of statement");
  }

  if (firstNumber.type !== "number") {
    throw new Error(`Expected number got ${firstNumber.type}`);
  }

  // Check to see if next non whitespace past first number is a comma
  // If it is, and that is followed by another number
  // We are likely dealing with a ClickHouse LIMIT <offset_number>,<limit_number> syntax
  const possibleComma = nextNonCommentNonWhitespace(
    tokens,
    firstNumber.index + 1
  );

  // If next token doesn't exist, or is not a comma
  // return the first number as it is the limit
  if (!possibleComma) {
    return firstNumber;
  }

  if (possibleComma.type !== "comma") {
    return firstNumber;
  }

  // Since we are dealing with "LIMIT <number> <comma>" find next number,
  // as it will be the actual limit for ClickHouse syntax
  const secondNumber = nextNonCommentNonWhitespace(
    tokens,
    possibleComma.index + 1
  );

  if (!secondNumber) {
    throw new Error("Unexpected end of statement");
  }

  if (secondNumber.type !== "number") {
    throw new Error(`Expected number got ${secondNumber.type}`);
  }

  return secondNumber;
}

/**
 * Adds limit to query that does not have it
 * @param {*} queryTokens
 * @param {*} statementKeywordIndex
 * @param {*} targetParenLevel
 * @param {*} limit
 */
function add(queryTokens, statementKeywordIndex, targetParenLevel, limit) {
  // Limit was not found, so figure out where it should be inserted
  // If last keyword is offset, need to put limit before that
  // If not offset, put limit at end, before terminator if present
  const insertBeforeToken = findParenLevelToken(
    queryTokens,
    statementKeywordIndex,
    (token) =>
      token.type === "keyword" &&
      (token.value === "offset" || token.value === "for")
  );
  if (insertBeforeToken) {
    const firstHalf = queryTokens.slice(0, insertBeforeToken.index);
    const secondhalf = queryTokens.slice(insertBeforeToken.index);
    return [
      ...firstHalf,
      createToken.keyword("limit"),
      createToken.singleSpace(),
      createToken.number(limit),
      createToken.singleSpace(),
      ...secondhalf,
    ];
  }

  // If there is a terminator add it just before
  const terminatorToken = findParenLevelToken(
    queryTokens,
    statementKeywordIndex,
    (token) => token.type === "terminator"
  );
  if (terminatorToken) {
    const firstHalf = queryTokens.slice(0, terminatorToken.index);
    const secondhalf = queryTokens.slice(terminatorToken.index);
    return [
      ...firstHalf,
      createToken.singleSpace(),
      createToken.keyword("limit"),
      createToken.singleSpace(),
      createToken.number(limit),
      ...secondhalf,
    ];
  }

  // No terminator. Append to end
  // skipping past any trailing comments, whitespace, terminator
  const targetIndex = findLimitInsertionIndex(queryTokens, targetParenLevel);
  const firstHalf = queryTokens.slice(0, targetIndex);
  const secondhalf = queryTokens.slice(targetIndex);
  return [
    ...firstHalf,
    createToken.singleSpace(),
    createToken.keyword("limit"),
    createToken.singleSpace(),
    createToken.number(limit),
    ...secondhalf,
  ];
}

exports.has = has;
exports.add = add;
